@@ -30,7 +30,7 @@ from subprocess import PIPE
 def getVersion():
     return version('appfire-connect-sdk')
 
-service_modules = ["ac-core", "ac-allowlist", "ac-automation", "ac-macro-security", "ac-migrations", "ac-proxy", "ac-utils"]
+service_modules = ["ac-core", "ac-allowlist", "ac-app-services", "ac-automation", "ac-macro-security", "ac-migrations", "ac-proxy", "ac-utils", "ac-data-engine-api"]
 
 CGREY = '\33[90m'
 CGREEN = '\033[92m'
@@ -59,9 +59,12 @@ CGREEN = '\033[92m'
 @click.option('--log-group','-n', help="AWS Log Group name", default="")
 @click.option('--starting-hour','-sh', help="Number of hours to go back for the start of tailing log entries for an AWS Log Group, the default(0) is the current time.", default=0)
 @click.option('--disassociate-cname-flag','-dcf', help="Disassociate CNAME to avoid CNAME conflict error", default="False")
+@click.option('--multi-region','-mr', help="Multi-region deployment", default="False")
+@click.option('--secondary-regions','-sr', help="List of secondary regions for multi-region deployment, comma seperated list", default="")
+@click.option('--brands','-bs', help="Brands required for certificates and domains", default="bobswift,wittified,bolo,botron,mohami")
 @click.argument('command')
 @click.help_option("--help", "-h")
-def process(verbose, command, profile, env, stage, stack, app_suffix, build, api_domain, api_stage, app_api_domain, require_approval, tag, bitbucket_username, bitbucket_password, bitbucket_branch, bitbucket_tag_value, aws_home, disable_clone_prompt, version, filter, log_group, starting_hour, disassociate_cname_flag):
+def process(verbose, command, profile, env, stage, stack, app_suffix, build, api_domain, api_stage, app_api_domain, require_approval, tag, bitbucket_username, bitbucket_password, bitbucket_branch, bitbucket_tag_value, aws_home, disable_clone_prompt, version, filter, log_group, starting_hour, disassociate_cname_flag, multi_region, secondary_regions, brands):
     """
     Gathers information related to deployment and deploys the CDK stack
 
@@ -73,7 +76,7 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
       build:dev     Build the app with webpack-dev.config.js config
         Options:  --profile, --env, --stage, --stack, --api-domain, --api-stage
       deploy        Build and deploy the specified stack
-        Options:  --profile, --stack, --tag, --aws-home
+        Options:  --profile, --stack, --tag, --aws-home, --multi-region, --secondary-regions
       diff          Diff the local stack with deployed stack
         Options:  --profile
       docker-build  Docker build the service module and push image to AWS DTS ECR
@@ -290,39 +293,41 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         #print(domain.split("."))
         app_suffix = 'green' if app_suffix == 'green' else ''
         
-        
         #generate_base_url(logger, "web/static/atlassian-connect.json", app_suffix, domain, env)
         #generate_base_url(logger, "atlassian-connect.json", app_suffix, domain, env)
 
         logger.debug("overriding cdk.json with supplied arguments")
         logger.info("AWS home=" + aws_home)
-        generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand, app_name, disassociate_cname_flag)
+
+        region_based_urls = get_region_based_urls(logger)
+
+        generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand, app_name, disassociate_cname_flag, region_based_urls)
         if stack == 'core':
-            deploy_core(logger,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_core(logger,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions, brands)
         elif stack == 'biz-service':
             deploy_biz_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
         elif stack == 'app-service':
-            deploy_app_services(logger,app_domain,brand,env,aws_profile)
+            deploy_service_module(logger,"ac-app-services",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions, brands)
         elif stack == 'module-service':
-            deploy_module_services(logger,app_name,api_stage,app_domain,brand,env,app_suffix,aws_profile)
+            deploy_service_module(logger,"ac-module-services",False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-allowlist':
-            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-automation':
-            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-core':
-            deploy_service_module(logger,stack,False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-data-engine-api':
-            deploy_service_module(logger,"ac-data-engine-api",False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,"ac-data-engine-api",False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-datastore':
-            deploy_service_module(logger,"ac-datastore",False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,"ac-datastore",False,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-macro-security':
-            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-migrations':
-            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-proxy':
-            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         elif stack == 'ac-utils':
-            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag)
+            deploy_service_module(logger,stack,True,app_name,api_stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions)
         else:
             if not aws_profile: checkCDKEnvVariables()
             ## Run build by default. Escape if false
@@ -375,18 +380,18 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
             profile_arg=" --profile "+ aws_profile if aws_profile else ""
             ), raise_error=True)
 
-def deploy_core(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
+def deploy_core(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions, brands):
     if deploy_biz_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile) :
-        if deploy_app_services(logger,app_domain,brand,env,aws_profile) :
-            if deploy_service_module(logger,"ac-core",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                if deploy_service_module(logger,"ac-datastore",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                    if deploy_service_module(logger,"ac-allowlist",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                        if deploy_service_module(logger,"ac-automation",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                            if deploy_service_module(logger,"ac-macro-security",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                                if deploy_service_module(logger,"ac-migrations",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                                    if deploy_service_module(logger,"ac-proxy",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                                        if deploy_service_module(logger,"ac-utils",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
-                                            if deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
+        if deploy_service_module(logger,"ac-app-services",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions, brands):
+            if deploy_service_module(logger,"ac-core",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions, brands):
+                if deploy_service_module(logger,"ac-datastore",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                    if deploy_service_module(logger,"ac-allowlist",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                        if deploy_service_module(logger,"ac-automation",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                            if deploy_service_module(logger,"ac-macro-security",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                                if deploy_service_module(logger,"ac-migrations",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                                    if deploy_service_module(logger,"ac-proxy",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                                        if deploy_service_module(logger,"ac-utils",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
+                                            if deploy_service_module(logger,"ac-module-services",False,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions):
                                                 print('Core services deployed successfully.')
 
 def deploy_biz_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
@@ -424,22 +429,10 @@ def deploy_app_services(logger,app_domain,brand,env,aws_profile):
 
     return True
 
-def deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
-    if (not os.path.isfile('./node_modules/ac-module-services/bin/ac-module-services.js')):
-        logger.info('ac-module-services dependency is not installed. Installing latest ac-module-services.')
-        shell(logger,"npm install ac-module-services --save-dev",raise_error=True)
-
-    ## Deploying ac-module-services
-    shell(logger, "cdk deploy '*' --app 'npx ts-node ./node_modules/ac-module-services/bin/ac-module-services.js " + env + " " + app_domain + " " + stage + "'" +  " {require_approval_arg} {profile_arg}".format(
-                require_approval_arg="--require-approval never",
-                profile_arg="--profile "+ aws_profile if aws_profile else ""
-            ), raise_error=True)
-
-    return True
-
-def deploy_service_module(logger,service_module,module_services_deploy,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag):
+def deploy_service_module(logger,service_module,module_services_deploy,app_name,stage,app_domain,brand,env,app_suffix,aws_home,aws_profile,registry_url,tag,multi_region,secondary_regions,brands=None):
     logger.info('Deploying ' + service_module)
-
+    logger.info("Multi-region : {} secondary regions: {}".format(multi_region,secondary_regions))
+   
     if not aws_profile: checkCDKEnvVariables()
 
     # check to see if a tags yml file exist for the service modules
@@ -463,12 +456,16 @@ def deploy_service_module(logger,service_module,module_services_deploy,app_name,
     if returnCode == 1:
         return False
 
-    cdk_deploy = generate_cdk_deploy(aws_home, aws_profile, os.environ.get("CDK_DEPLOY_ACCOUNT"), os.environ.get("CDK_DEPLOY_REGION"))
+    cdk_deploy = generate_cdk_deploy(aws_home, aws_profile, os.environ.get("CDK_DEPLOY_ACCOUNT"), os.environ.get("CDK_DEPLOY_REGION"), multi_region, secondary_regions)
 
     if service_module == "ac-automation":
         returnCode = shell(logger,cdk_deploy + " -e STAGE=" + stage  + " " + service_module_image,raise_error=False)
     elif service_module == "ac-core":
         returnCode = shell(logger,cdk_deploy + " -e ENV=" + env  + " -e DOMAIN=" + app_domain + " -e APP_SUFFIX=" + app_suffix + " " + service_module_image,raise_error=False)
+    elif service_module == "ac-module-services":
+        returnCode = shell(logger,cdk_deploy + " -e ENV=" + env  + " -e DOMAIN=" + app_domain + " -e STAGE=" + stage + " " + service_module_image,raise_error=False)
+    elif service_module == "ac-app-services":
+        returnCode = shell(logger,cdk_deploy + " -e ENV=" + env  + " -e DOMAIN=" + app_domain  + " -e BRANDS=" + brands + " -e STAGE=" + stage + " " + service_module_image,raise_error=False)
     else: 
         returnCode = shell(logger,cdk_deploy + " -e ENV=" + env + " -e STAGE=" + stage + " " + service_module_image,raise_error=False)
 
@@ -477,11 +474,19 @@ def deploy_service_module(logger,service_module,module_services_deploy,app_name,
 
     if module_services_deploy == True:
         logger.info('Deploying ac-module-services')
-        deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
+        service_module_image = generate_image_url("ac-module-services",registry_url,tag)
+        returnCode = shell(logger,"docker pull " + service_module_image,raise_error=False)
+        if returnCode == 1:
+            return False
+
+        returnCode = shell(logger,cdk_deploy + " -e ENV=" + env  + " -e DOMAIN=" + app_domain + " -e STAGE=" + stage + " " + service_module_image,raise_error=False)
+
+        if returnCode == 1:
+            return False
 
     return True
 
-def generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand, app_name, disassociate_cname_flag):
+def generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand, app_name, disassociate_cname_flag, region_based_urls):
     '''dynamically prepare cdk.json based on command line arguments'''
     
     try:
@@ -491,7 +496,8 @@ def generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand,
                 if item == 'app':
                     json_data[
                         item] = "npx ts-node ./node_modules/ac-app-dist/bin/ac-app-dist.js " + app_name + " " + stage \
-                                + " " + app_domain + " client/dist " + brand + " " + env + " false '" + app_suffix + "' " + disassociate_cname_flag
+                                + " " + app_domain + " client/dist " + brand + " " + env + " false '" + app_suffix + "' " + disassociate_cname_flag \
+                                + " " + region_based_urls
         with open('cdk.json', 'w') as file:
             json.dump(json_data, file, indent=2)
     except IOError:
@@ -567,24 +573,28 @@ def dts_ecr_login(logger, profile):
 def generate_image_url(service_module,registry_url,tag):
     return registry_url + "/" + service_module + ":" + tag
 
-def generate_cdk_deploy(aws_home,aws_profile,cdk_deploy_account,cdk_deploy_region):
+def generate_cdk_deploy(aws_home,aws_profile,cdk_deploy_account,cdk_deploy_region,multi_region,secondary_regions):
 
     # Build a CDK deploy string based either on AWS_PROFILE or AWS ENV Args
     if aws_profile:
-        cdk_deploy = "docker run {aws_home_arg} {cdk_deploy_account_arg} {cdk_deploy_region_arg} {profile_arg}".format(
+        cdk_deploy = "docker run {aws_home_arg} {cdk_deploy_account_arg} {cdk_deploy_region_arg} {profile_arg} {multi_region_arg} {secondary_regions_arg}".format(
             aws_home_arg=" -v " + aws_home + ":/appfire/.aws",
             cdk_deploy_account_arg="-e CDK_DEPLOY_ACCOUNT="+cdk_deploy_account,
             cdk_deploy_region_arg="-e CDK_DEPLOY_REGION="+ cdk_deploy_region,
-            profile_arg="-e AWS_PROFILE="+ aws_profile)
+            profile_arg="-e AWS_PROFILE="+ aws_profile,
+            multi_region_arg="-e MUTLI_REGION="+ multi_region,
+            secondary_regions_arg="-e SECONDARY_REGIONS="+ secondary_regions)
     else:
         checkCDKEnvVariables()
 
-        cdk_deploy = "docker run {cdk_deploy_account_arg} {cdk_deploy_region_arg} {aws_default_region_arg} {aws_access_key_id_arg} {aws_secret_access_key_arg}".format(
+        cdk_deploy = "docker run {cdk_deploy_account_arg} {cdk_deploy_region_arg} {aws_default_region_arg} {aws_access_key_id_arg} {aws_secret_access_key_arg} {multi_region_arg} {secondary_regions_arg}".format(
             aws_access_key_id_arg="-e AWS_ACCESS_KEY_ID="+ os.getenv("AWS_ACCESS_KEY_ID"),
             aws_secret_access_key_arg="-e AWS_SECRET_ACCESS_KEY="+ os.getenv("AWS_SECRET_ACCESS_KEY"),
             aws_default_region_arg="-e AWS_DEFAULT_REGION="+ cdk_deploy_region,
             cdk_deploy_account_arg="-e CDK_DEPLOY_ACCOUNT="+cdk_deploy_account,
-            cdk_deploy_region_arg="-e CDK_DEPLOY_REGION="+ cdk_deploy_region)
+            cdk_deploy_region_arg="-e CDK_DEPLOY_REGION="+ cdk_deploy_region,
+            multi_region_arg="-e MUTLI_REGION="+ multi_region,
+            secondary_regions_arg="-e SECONDARY_REGIONS="+ secondary_regions)
 
     print(f"Generated CDK={cdk_deploy}")
     return cdk_deploy
@@ -668,6 +678,7 @@ def checkForKeyboardInterrupt(value):
         raise KeyboardInterrupt
 
 def isValidServiceModuleGitRepo(repoPath):
+
     if repoPath == "":
         return ""
 
@@ -788,6 +799,21 @@ def do_filter_log_events(client, filter_logs_events_kwargs):
                 event_ids_per_timestamp
             )
             time.sleep(SLEEP)
+
+def get_region_based_urls(logger):
+    region_base_urls = ""
+    try:
+        with open('atlassian-connect.json', 'r') as file:
+            json_data = json.load(file)
+
+            if 'regionBaseUrls' in json_data.keys():
+                for i in json_data['regionBaseUrls']:
+                    region_base_urls = region_base_urls + "," if region_base_urls else ""
+                    region_base_urls = region_base_urls + i
+
+            return region_base_urls
+    except IOError:
+        shell(logger, "atlassian-connect.json not found!")
 class ShellCommandFailed(Exception):
     """ Executing a shell command failed """
 
