@@ -47,6 +47,7 @@ def process(verbose):
         atlApp = questionary.select(
             "Atlassian App?",
             choices=[
+                'bitbucket',
                 'confluence',
                 'jira'
             ]).ask()  # returns value of selection
@@ -61,12 +62,17 @@ def process(verbose):
         checkForKeyboardInterrupt(cloneBy)
 
         gitLink = "git@bitbucket.org:appfire/appfire-connect-app-template.git"
+        if atlApp == "bitbucket":
+            gitLink = "git@bitbucket.org:appfire/fuse-bb-app-template.git"
+
         if cloneBy == 'https':
             userName = questionary.text(
                 "Enter your git user name:"
             ).ask()  # returns the name of the user in Bitbucket
             checkForKeyboardInterrupt(userName)
             gitLink = "https://{}@bitbucket.org/appfire/appfire-connect-app-template.git".format(userName)
+            if atlApp == "bitbucket":
+                gitLink = "https://{}@bitbucket.org/appfire/fuse-bb-app-template.git".format(userName)
 
     except KeyboardInterrupt:
         sys.exit(1)
@@ -78,7 +84,7 @@ def process(verbose):
     def git(*args):
         return subprocess.check_call(['git'] + list(args))
 
-    git("clone", gitLink, "--single-branch")
+    git("clone", gitLink, "appfire-connect-app-template", "--single-branch")
 
     dirName = "{}-{}-connect".format(atlApp, appName)
 
